# Created by Noah Kantrowitz on 2007-04-03.
# Modified by Iker Jimenez on 2008-12-18.
# Copyright (c) 2007 Noah Kantrowitz. All rights reserved.
# Copyright (c) 2008 Iker Jimenez. All rights reserved.

from trac.core import *
from trac.web.chrome import ITemplateProvider
from trac.ticket.api import TicketSystem
from trac.util import sorted
from trac.admin import IAdminPanelProvider

from api import HideValsSystem


class HideValsAdminModule(Component):
    """WebAdmin sub-page for configuring the TracHideVals plugins."""

    implements(IAdminPanelProvider, ITemplateProvider)

    # IAdminPanelProvider methods
    def get_admin_panels(self, req):
        if req.perm.has_permission('TRAC_ADMIN'):
            for field in TicketSystem(self.env).get_ticket_fields():
                if 'options' in field:
                    yield 'hidevals', 'Hide Values', field['name'], field['label']

    def render_admin_panel(self, req, cat, page, path_info):
        db = self.env.get_db_cnx()
        cursor = db.cursor()
        field = dict([(field['name'], field)
                      for field in TicketSystem(self.env).get_ticket_fields()])[page]
        cursor.execute(
            'SELECT sid, value FROM hidevals WHERE field = %s', (field['name'],))
        values = cursor.fetchall()
        enabled = field['name'] not in HideValsSystem(self.env).dont_filter
        if req.method == 'POST':
            if req.args.get('add'):
                group = req.args['group']
                value = req.args['value']
                if (group, value) not in values:
                    cursor.execute(
                        'INSERT INTO hidevals (sid, field, value) VALUES (%s, %s, %s)', (group, field['name'], value))
                    db.commit()
            elif req.args.get('remove'):
                sel = req.args.getlist('sel')
                for val in sel:
                    group, value = val.split('#', 1)
                    cursor.execute(
                        'DELETE FROM hidevals WHERE sid = %s AND field = %s AND value = %s', (group, field['name'], value))
                    db.commit()
            elif req.args.get('toggle'):
                new_val = HideValsSystem(self.env).dont_filter[:]
                if enabled:
                    new_val.append(field['name'])
                else:
                    new_val.remove(field['name'])
                self.config.set('hidevals', 'dont_filter',
                                ', '.join(sorted(new_val)))
                self.config.save()

            req.redirect(req.href.admin(cat, page))

        data = {'field': field,
                'values': [{'group': g, 'value': v} for g, v in values],
                'enabled': enabled}
        return 'admin_hidevals.html', data

    # ITemplateProvider methods
    def get_templates_dirs(self):
        from pkg_resources import resource_filename
        return [resource_filename(__name__, 'templates')]

    def get_htdocs_dirs(self):
        return []



# Created by Noah Kantrowitz on 2007-04-02.
# Modified by Iker Jimenez on 2008-12-18. Based on a patch submited to
# http://trac-hacks.org/ticket/1920 by user kkas.
# Copyright (c) 2007 Noah Kantrowitz. All rights reserved.
# Copyright (c) 2008 Iker Jimenez. All rights reserved.

from trac.core import Component, implements
from trac.web.api import IRequestFilter

try:
    set = set
except NameError:
    from sets import Set as set

from api import HideValsSystem


class HideValsFilter(Component):
    """A filter to hide certain ticket field values."""

    implements(IRequestFilter)

    # IRequestFilter methods
    def pre_process_request(self, req, handler):
        return handler

    def post_process_request(self, req, template, data, content_type):
        if 'TRAC_ADMIN' in req.perm or \
                'TICKET_HIDEVALS' not in req.perm or \
                not req.path_info.startswith(('/newticket', '/ticket',
                                              '/query')) or \
                (template, data, content_type) == (None, None, None):
            # TRAC_ADMIN would have the filterer permissions by inheritance
            return template, data, content_type
        else:
            visible_fields = HideValsSystem(self.env).visible_fields(req)
            self.log.debug("visible_fields: %s", visible_fields)
            dont_filter = set(HideValsSystem(self.env).dont_filter)
            self.log.debug("dont_filter: %s", dont_filter)
            to_delete = []
            fields = data['fields']

            def filter_field(field):
                if field['options']:
                    opts = field['options']
                    valid_opts = visible_fields[field['name']]
                    opts_to_delete = []
                    for opt in opts:
                        if opt not in valid_opts:
                            opts_to_delete.append(opt)

                    for opt in opts_to_delete:
                        self.log.debug(
                            "HideValsFilter: '%s' option removed "
                            "from '%s' field", opt, field['name'])
                        opts.remove(opt)
                elif field['optgroups']:
                    groups = field['optgroups']
                    valid_opts = visible_fields[field['name']]
                    opts_to_delete = []
                    for grp in groups:
                        for opt in grp['options']:
                            if opt not in valid_opts:
                                opts_to_delete.append(opt)

                    for grp in groups:
                        for opt in opts_to_delete:
                            if opt in grp['options']:
                                self.log.debug(
                                    "HideValsFilter: '%s' option removed "
                                    "from '%s' field",
                                    opt, field['name'])
                                grp['options'].remove(opt)

            if req.path_info.startswith('/newticket') or \
                    req.path_info.startswith('/ticket'):
                for field in fields:
                    if (field.get('options') or field.get('optgroups')) and \
                            field['name'] not in dont_filter:
                        if field['name'] in visible_fields:
                            filter_field(field)
                        else:
                            # If there are no values for this user, remove
                            # the field entirely
                            # NOTE: Deleting in place screws up the
                            # iteration, so do it all afterwards.
                            to_delete.append(field)
                for field in to_delete:
                    self.env.log.debug(
                        "HideValsFilter: '%s' field removed", field['name'])
                    fields.remove(field)
            elif req.path_info.startswith('/query'):
                for field in fields.itervalues():
                    if (field.get('options') or field.get('optgroups')) and \
                            field['name'] not in dont_filter:
                        if field['name'] in visible_fields:
                            filter_field(field)
                        else:
                            # If there are no values for this user, remove
                            # the field entirely
                            # NOTE: Deleting in place screws up the
                            # iteration, so do it all afterwards.
                            to_delete.append(field['name'])
                for field_name in to_delete:
                    self.log.debug(
                        "HideValsFilter: '%s' field removed", field_name)
                    del fields[field_name]
            return template, data, content_type



#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-

from setuptools import setup

setup(
    name='TracHideVals',
    version='1.2.1',
    packages=['hidevals'],
    package_data={'hidevals': [
        'templates/*.html', 'htdocs/*.js', 'htdocs/*.css']},

    author='Noah Kantrowitz, Iker Jimenez',
    author_email='noah@coderanger.net, iker.jimenez@gmail.com',
    description='Hide ticket option values from certain users.',
    license='BSD',
    keywords='trac plugin',
    url='http://trac-hacks.org/wiki/HideValsPlugin',
    classifiers=[
        'Framework :: Trac',
    ],

    install_requires=[],

    entry_points={
        'trac.plugins': [
            'hidevals.filter = hidevals.filter',
            'hidevals.api = hidevals.api',
            'hidevals.admin = hidevals.admin',
        ]
    },
)
